<!DOCTYPE html>
<html lang="en-US">
<head>
    <meta charset="UTF-8" />
    <title>Thank you for signing up!</title>
</head>
<body>
<p>
Verify your email address by clicking on this link:<br>
<a href="http://127.0.0.1:8000/signup/verify/?code={{ code }}">http://127.0.0.1:8000/signup/verify/?code={{ code }}</a>
</p>
</body>
</html>
