<!DOCTYPE html>
<html lang="en-US">
<head>
    <meta charset="UTF-8" />
    <title>Notify Previous Email Address</title>
</head>
<body>
<p>
A request has been made to change the email address on your account.<br>
The email address will be changed, if confirmed.
</p>
</body>
</html>
